# Copyright 2013 Cloudera Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


"""
This test module simply wraps _dbapi20_tests.py, which ensure that our
PEP 249 implementation is compliant.
"""


from __future__ import absolute_import, print_function

import pytest

import impala.dbapi
from impala.tests.util import ImpylaTestEnv
from impala.util import (
    _random_id, force_drop_impala_database, force_drop_hive_database)
# must import the module, rather than the class, per comment in module
from impala.tests import _dbapi20_tests


ENV = ImpylaTestEnv()
tmp_db = _random_id('tmp_impyla_dbapi_')
hive = ENV.auth_mech == 'PLAIN'


@pytest.mark.connect
class ImpalaDBAPI20Test(_dbapi20_tests.DatabaseAPI20Test):
    driver = impala.dbapi
    connect_kw_args = {'host': ENV.host,
                       'port': ENV.port,
                       'auth_mechanism': ENV.auth_mech,
                       'database': tmp_db}

    ddl1 = 'create table {0}booze (name string)'.format(
        _dbapi20_tests.DatabaseAPI20Test.table_prefix)
    ddl2 = 'create table {0}barflys (name string)'.format(
        _dbapi20_tests.DatabaseAPI20Test.table_prefix)

    @classmethod
    def setUpClass(cls):
        con = cls.driver.connect(host=ENV.host, port=ENV.port,
                                 auth_mechanism=ENV.auth_mech)
        cur = con.cursor()
        cur.execute('CREATE DATABASE {0}'.format(tmp_db))
        cur.close()
        con.close()

    @classmethod
    def tearDownClass(cls):
        con = cls.driver.connect(host=ENV.host, port=ENV.port,
                                 auth_mechanism=ENV.auth_mech)
        cur = con.cursor()
        if hive:
            force_drop_hive_database(cur, tmp_db)
        else:
            force_drop_impala_database(cur, tmp_db)
        cur.close()
        con.close()

    def test_nextset(self):
        pass

    def test_setoutputsize(self):
        pass

    # The following tests fail against Hive. We override them so we can skip on
    # Hive, and otherwise execute the superclass version.

    HIVE_STRING_ESCAPE_ERROR = 'HIVE-11723'

    @pytest.mark.skipif(hive, reason=HIVE_STRING_ESCAPE_ERROR)
    def test_execute(self):
        super(ImpalaDBAPI20Test, self).test_execute()

    @pytest.mark.skipif(hive, reason=HIVE_STRING_ESCAPE_ERROR)
    def test_executemany(self):
        super(ImpalaDBAPI20Test, self).test_executemany()

    @pytest.mark.skipif(hive, reason=HIVE_STRING_ESCAPE_ERROR)
    def test_setinputsizes(self):
        super(ImpalaDBAPI20Test, self).test_setinputsizes()

    @pytest.mark.skipif(hive, reason=HIVE_STRING_ESCAPE_ERROR)
    def test_setoutputsize_basic(self):
        super(ImpalaDBAPI20Test, self).test_setoutputsize_basic()



#
# Autogenerated by Thrift Compiler (0.9.3)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py:new_style
#

from thrift.Thrift import TType, TMessageType, TException, TApplicationException
import impala._thrift_gen.ErrorCodes.ttypes


from thrift.transport import TTransport
from thrift.protocol import TBinaryProtocol, TProtocol
try:
  from thrift.protocol import fastbinary
except:
  fastbinary = None



class TStatus(object):
  """
  Attributes:
   - status_code
   - error_msgs
  """

  thrift_spec = (
    None, # 0
    (1, TType.I32, 'status_code', None, None, ), # 1
    (2, TType.LIST, 'error_msgs', (TType.STRING,None), None, ), # 2
  )

  def __init__(self, status_code=None, error_msgs=None,):
    self.status_code = status_code
    self.error_msgs = error_msgs

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.I32:
          self.status_code = iprot.readI32()
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.LIST:
          self.error_msgs = []
          (_etype3, _size0) = iprot.readListBegin()
          for _i4 in xrange(_size0):
            _elem5 = iprot.readString()
            self.error_msgs.append(_elem5)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('TStatus')
    if self.status_code is not None:
      oprot.writeFieldBegin('status_code', TType.I32, 1)
      oprot.writeI32(self.status_code)
      oprot.writeFieldEnd()
    if self.error_msgs is not None:
      oprot.writeFieldBegin('error_msgs', TType.LIST, 2)
      oprot.writeListBegin(TType.STRING, len(self.error_msgs))
      for iter6 in self.error_msgs:
        oprot.writeString(iter6)
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    if self.status_code is None:
      raise TProtocol.TProtocolException(message='Required field status_code is unset!')
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.status_code)
    value = (value * 31) ^ hash(self.error_msgs)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.iteritems()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)
